#include "gridstore.h"
#include <stdio.h>
#include <stdlib.h>

typedef struct {
	const GSChar* name; // Device name
	double amperage; // Amperage value
	int count; 
	GSTimestamp timestamp; 
} Device; // Static Column Schema in GridDB C API

// Create A Column Schema from the C-Struct
// The 'name' column will be the row-key
GS_STRUCT_BINDING(Device,
	GS_STRUCT_BINDING_KEY(name,GS_TYPE_STRING)
	GS_STRUCT_BINDING_ELEMENT(amperage,GS_TYPE_DOUBLE)
	GS_STRUCT_BINDING_ELEMENT(count,GS_TYPE_INTEGER)
	GS_STRUCT_BINDING_ELEMENT(timestamp,GS_TYPE_TIMESTAMP));

void sample(const char* host, const char* port, const char* clusterName, const char* user, const char* password){
	GSGridStore* gridstore;
	const GSPropertyEntry properties[] = {
		{"notificationAddress",host},
		{"notificationPort",port},
		{"clusterName",clusterName},
		{"user",user},
		{"password",password}
	}; 
	size_t propertyCount = sizeof(properties) / sizeof(*properties);
	gsGetGridStore(gsGetDefaultFactory(),properties,propertyCount,&gridstore); // Connect to a GridDB cluster

	GSCollection* collection; // Insert Collection container with Device-schema into GridDB
	gsPutCollection(gridstore,"ampCollection151",GS_GET_STRUCT_BINDING(Device),NULL,GS_FALSE,&collection);

	gsCreateIndex(collection,"name",GS_INDEX_FLAG_DEFAULT); // Create a 'name' index 

	// Create Device to be used as Row-Object for 'ampCollection151'
	Device insertionDevice;
	insertionDevice.name = "ampMeter12"; // Set 'name' column
	insertionDevice.amperage = 1.45; // Set 'amperage' column
	insertionDevice.count = 3; // Set 'count' column
	insertionDevice.timestamp = gsCurrentTime(); // Set 'timestamp' column with current time

	gsPutRow(collection,NULL,&insertionDevice,NULL); // Insert Device as a row-object into Collection container

	GSQuery* query;
	GSRowSet* rowSet;

	// Create, issue, and fetch a query where the 'count' column equals 3
	gsQuery(collection,"select * where count=3",&query);
	gsFetch(query,GS_FALSE,&rowSet);

	Device device; // Row-object to fetch 
	printf("Fetching Rows from ampCollection151 using the Device Struct\n");
	while(gsHasNextRow(rowSet)){
		gsGetNextRow(rowSet,&device);
		GSChar timeStr[GS_TIME_STRING_SIZE_MAX]; 
		gsFormatTime(device.timestamp,timeStr,sizeof(timeStr)); // Format timestamp into readable format
		printf("Device in ampCollection151: ");
		printf("name=%s",device.name);
		printf(" amperage=%.2lf",device.amperage);
		printf(" count=%d",device.count);
		printf(" timestamp=%s\n",timeStr);
	}

	// Insert another device-row
	Device secondDevice;
	secondDevice.name = "BLUESEAMETER555";
	secondDevice.amperage = 2.43;
	secondDevice.count = 5;
	secondDevice.timestamp = gsCurrentTime();

	gsPutRow(collection,NULL,&secondDevice,NULL);

	Device nextDevice;
	GSQuery* generalQuery;
	GSRowSet* generalRowSet;

	gsQuery(collection, "select * where (SUBSTRING(name,8,3)='r12' OR name LIKE '%LUE%R55_') and CHAR_LENGTH(name) > 4", &generalQuery);
	gsFetch(generalQuery,GS_FALSE,&generalRowSet);
	printf("Performing a String Operation on Collection ampCollection151 in TQL\n\n");
	while(gsHasNextRow(generalRowSet)){
		GSChar timeString[GS_TIME_STRING_SIZE_MAX];
		gsGetNextRow(generalRowSet,&nextDevice); // Obtain  row object
		gsFormatTime(nextDevice.timestamp,timeString,sizeof(timeString)); // Format timestamp number

		printf("Row in collection ampMeter151: Name=%s",nextDevice.name);
		printf(" Amperage = %.2lf",nextDevice.amperage);
		printf(" Count = %d",nextDevice.count);
		printf(" Timestamp = %s\n",timeString);
	}

	gsDropCollection(gridstore,"ampCollection151");
	gsCloseGridStore(&gridstore,GS_TRUE);  // Clean up and disconnect from GridDB
}



int main(int argc, char** argv){
	if(argc < 6){
		printf("Need to enter host, port, clusterName, username, password to run this sample program\n");
		return 1;
	} else {
		sample(argv[1],argv[2],argv[3],argv[4],argv[5]);
	}
	return 0;
}